!! Copyright INRIA. Contributors : Julien DIAZ and Abdelaaziz EZZIANI
!! 
!! Julien.Diaz@inria.fr and Abdelaaziz.Ezziani@univ-pau.fr
!! 
!! This software is a computer program whose purpose is to
!! compute the analytical solution of problems of waves propagation in two 
!! layered media such as
!! - acoustic/acoustic
!! - acoustic/elastodynamic
!! - acoustic/porous
!! - porous/porous,
!! based on the Cagniard-de Hoop method.
!! 
!! This software is governed by the CeCILL license under French law and
!! abiding by the rules of distribution of free software.  You can  use, 
!! modify and/ or redistribute the software under the terms of the CeCILL
!! license as circulated by CEA, CNRS and INRIA at the following URL
!! "http://www.cecill.info". 
!! 
!! As a counterpart to the access to the source code and  rights to copy,
!! modify and redistribute granted by the license, users are provided only
!! with a limited warranty  and the software's author,  the holder of the
!! economic rights,  and the successive licensors  have only  limited
!! liability. 
!! 
!! In this respect, the user's attention is drawn to the risks associated
!! with loading,  using,  modifying and/or developing or reproducing the
!! software by the user in light of its specific status of free software,
!! that may mean  that it is complicated to manipulate,  and  that  also
!! therefore means  that it is reserved for developers  and  experienced
!! professionals having in-depth computer knowledge. Users are therefore
!! encouraged to load and test the software's suitability as regards their
!! requirements in conditions enabling the security of their systems and/or 
!! data to be ensured and,  more generally, to use and operate it in the 
!! same conditions as regards security. 
!! 
!! The fact that you are presently reading this means that you have had
!! knowledge of the CeCILL license and that you accept its terms.
!! ========================================================================

SUBROUTINE sub_reflex_aa
  Use m_phys  
  Use m_num
  Use m_source
  Use m_sismo
  Use m_const
  implicit none 
#include "m_interface.h90"
  integer I,J,K
  real*8 t0,r,x,ddt,src,t,tau,F,thead,green,dq,q,q0
  complex*16::pp,Coeff,k1, k2
  write(6,*) 'Computation of the reflected wave'
  F=1/V1**2
  DO I=1,Nx
     x=X1+(I-1)*dx
     r=dsqrt(x**2+(y+h)**2)
     t0=r/V1
     DO J=1,Nt
       t=T1+(j-1)*dt
       if (abs(x/r).gt.V1/V2) then
          !Computation of the head wave'
          thead=abs(y+h)*sqrt(1/V1**2-1/V2**2)+abs(x)/V2;
          if((t.gt.thead).and.(t-2.*tdelay.lt.t0)) then
             if (t.ge.t0) then
                if (t-2.*tdelay.gt.thead) then
                   dq=(pi/2.-asin((t-2.*tdelay)/t0))/Nint
                   q0=asin((t-2.*tdelay)/t0)
                else
                   dq=pi/2./Nint
                   q0=0
                end if
             else
                if (t-2.*tdelay.gt.thead) then
                   dq=(asin(t/t0)-asin((t-2.*tdelay)/t0))/Nint
                   q0=asin((t-2.*tdelay)/t0)
                else
                   dq=asin(t/t0)/Nint
                   q0=0
                end if
             end if
             DO  k=1,Nint
                q=q0+(k-1)*dq+dq/2.
                tau=t0*sin(q)
                src=source(t-tau)
                pp=-ima*(abs(y+h)/r**2*t0*cos(q)-abs(x)*tau/r**2)
                k1=zsqrt(pp**2+1/V1**2)
                k2=ima*zsqrt(-pp**2-1/V2**2)
                Coeff=rho2*k1-rho1*k2
                Coeff=Coeff/(rho2*k1+rho1*k2)/(2.*pi)*dq
                Ux(i,j)=Ux(i,j)-F*aimag(ima*pp*Coeff)*src/rho1
                Uy(i,j)=Uy(i,j)-F*aimag(k1*Coeff)*src/rho1
                P(i,j)=P(i,j)-F*aimag(Coeff)*src
             END DO
       end if
    end if
       if (t.gt.t0) then
          if (t-2.*tdelay.gt.t0) then
             dq=(acosh(t/t0)-acosh((t-2.*tdelay)/t0))/Nint
             q0=acosh((t-2.*tdelay)/t0)
          else
             dq=acosh(t/t0)/Nint
             q0=0
          end if
          do k=1,Nint
             q=q0+(k-1)*dq+dq/2.
             tau=t0*cosh(q)
             src=source(t-tau)
             pp=ima*tau*abs(x)/r**2+abs(y+h)/r**2*t0*sinh(q)
             k1=zsqrt(pp**2+1/V1**2)
             k2=zsqrt(pp**2+1/V2**2)
             Coeff=rho2*k1-rho1*k2
             Coeff=Coeff/(rho2*k1+rho1*k2)/(2*pi)*dq
             Ux(i,j)=Ux(i,j)+F*real(ima*pp*Coeff)*src/rho1
             k1=zsqrt(pp**2+1/V1**2)
             Uy(i,j)=Uy(i,j)+F*real(k1*Coeff)*src/rho1
             P(i,j)=P(i,j)+F*real(Coeff)*src
          end do
       end if
     END DO
  END DO
END SUBROUTINE sub_reflex_aa
