!! Copyright INRIA. Contributors : Julien DIAZ and Abdelaaziz EZZIANI
!! 
!! Julien.Diaz@inria.fr and Abdelaaziz.Ezziani@univ-pau.fr
!! 
!! This software is a computer program whose purpose is to
!! compute the analytical solution of problems of waves propagation in two 
!! layered media such as
!! - acoustic/acoustic
!! - acoustic/elastodynamic
!! - acoustic/porous
!! - porous/porous,
!! based on the Cagniard-de Hoop method.
!! 
!! This software is governed by the CeCILL license under French law and
!! abiding by the rules of distribution of free software.  You can  use, 
!! modify and/ or redistribute the software under the terms of the CeCILL
!! license as circulated by CEA, CNRS and INRIA at the following URL
!! "http://www.cecill.info". 
!! 
!! As a counterpart to the access to the source code and  rights to copy,
!! modify and redistribute granted by the license, users are provided only
!! with a limited warranty  and the software's author,  the holder of the
!! economic rights,  and the successive licensors  have only  limited
!! liability. 
!! 
!! In this respect, the user's attention is drawn to the risks associated
!! with loading,  using,  modifying and/or developing or reproducing the
!! software by the user in light of its specific status of free software,
!! that may mean  that it is complicated to manipulate,  and  that  also
!! therefore means  that it is reserved for developers  and  experienced
!! professionals having in-depth computer knowledge. Users are therefore
!! encouraged to load and test the software's suitability as regards their
!! requirements in conditions enabling the security of their systems and/or 
!! data to be ensured and,  more generally, to use and operate it in the 
!! same conditions as regards security. 
!! 
!! The fact that you are presently reading this means that you have had
!! knowledge of the CeCILL license and that you accept its terms.
!! ========================================================================

SUBROUTINE sub_transmit_aa
  Use m_phys  
  Use m_num
  Use m_source
  Use m_sismo
  Use m_const
  implicit none 
#include "m_interface.h90"
  integer I,J,K
  real*8 t0,x,ddt,src,t,tau,F
  complex*16::pp,Coeff,k1, k2,green
  write(6,*) 'Computation of the transmitted wave'
  F=1/V1**2
  DO I=1,Nx
     x=X1+(I-1)*dx
     t0=arrivaltime(h,x,y,V1,V2)
     DO J=1,Nt
       t=T1+(j-1)*dt
       if (t.gt.t0) then
          if (t-2.*tdelay.gt.t0) then
             ddt=2*tdelay/Nint
             do k=1,Nint
                tau=t-2*tdelay+(k-1)*ddt+ddt/2.
                src=source(t-tau)
                pp=path(-abs(x),y,h,V1,V2,tau)
                k1=zsqrt(pp**2+1/V1**2)
                k2=zsqrt(pp**2+1/V2**2)
                Coeff=rho2/(rho2*k1+rho1*k2)
                green=derivee(-abs(x),y,h,V1,V2,pp)/pi*ddt
                Ux(i,j)=Ux(i,j)+F*real(ima*pp*Coeff*green)*src/rho2
                Uy(i,j)=Uy(i,j)-F*real(k2*Coeff*green)*src/rho2
                P(i,j)=P(i,j)+F*real(Coeff*green)*src
             end do
          else
             ddt=(t-t0)/Nint
             tau=t0+ddt/2.
             src=source(t-tau)
             pp=path(-abs(x),y,h,V1,V2,tau);
             k1=zsqrt(pp**2+1/V1**2)
             k2=zsqrt(pp**2+1/V2**2)
             Coeff=rho2/(rho2*k1+rho1*k2)
             green=sqrt(tau**2-t0**2)*derivee(-abs(x),y,h,V1,V2,pp)*dacosh(1+ddt/t0)/pi
             Ux(i,j)=Ux(i,j)+F*real(ima*pp*Coeff*green)*src/rho2
             Uy(i,j)=Uy(i,j)-F*real(k2*Coeff*green)*src/rho2
             P(i,j)=P(i,j)+F*real(Coeff*green)*src
             do k=2,Nint
                tau=t0+(k-1)*ddt+ddt/2.
                src=source(t-tau)
                pp=path(-abs(x),y,h,V1,V2,tau)
                k1=zsqrt(pp**2+1/V1**2)
                k2=zsqrt(pp**2+1/V2**2)
                Coeff=rho2/(rho2*k1+rho1*k2)
                green=derivee(-abs(x),y,h,V1,V2,pp)/pi*ddt
                Ux(i,j)=Ux(i,j)+F*real(ima*pp*Coeff*green)*src/rho2
                Uy(i,j)=Uy(i,j)-F*real(k2*Coeff*green)*src/rho2
                P(i,j)=P(i,j)+F*real(Coeff*green)*src
          end do
       end if
       end if
     END DO
  END DO
END SUBROUTINE sub_transmit_aa
