!! Copyright INRIA. Contributors : Julien DIAZ and Abdelaaziz EZZIANI
!! 
!! Julien.Diaz@inria.fr and Abdelaaziz.Ezziani@univ-pau.fr
!! 
!! This software is a computer program whose purpose is to
!! compute the analytical solution of problems of waves propagation in two 
!! layered media such as
!! - acoustic/acoustic
!! - acoustic/elastodynamic
!! - acoustic/porous
!! - porous/porous,
!! based on the Cagniard-de Hoop method.
!! 
!! This software is governed by the CeCILL license under French law and
!! abiding by the rules of distribution of free software.  You can  use, 
!! modify and/ or redistribute the software under the terms of the CeCILL
!! license as circulated by CEA, CNRS and INRIA at the following URL
!! "http://www.cecill.info". 
!! 
!! As a counterpart to the access to the source code and  rights to copy,
!! modify and redistribute granted by the license, users are provided only
!! with a limited warranty  and the software's author,  the holder of the
!! economic rights,  and the successive licensors  have only  limited
!! liability. 
!! 
!! In this respect, the user's attention is drawn to the risks associated
!! with loading,  using,  modifying and/or developing or reproducing the
!! software by the user in light of its specific status of free software,
!! that may mean  that it is complicated to manipulate,  and  that  also
!! therefore means  that it is reserved for developers  and  experienced
!! professionals having in-depth computer knowledge. Users are therefore
!! encouraged to load and test the software's suitability as regards their
!! requirements in conditions enabling the security of their systems and/or 
!! data to be ensured and,  more generally, to use and operate it in the 
!! same conditions as regards security. 
!! 
!! The fact that you are presently reading this means that you have had
!! knowledge of the CeCILL license and that you accept its terms.
!! ========================================================================

SUBROUTINE sub_transmitpsi_ap
  Use m_phys  
  Use m_num
  Use m_source
  Use m_sismo
  Use m_const
  implicit none 
#include "m_interface.h90"
  integer I,J,K
  real*8 t0,x,ddt,src,t,tau,F,thead,tau0
  complex*16::pp,Coeff(4),kpsi2,pp0,green
  write(6,*) 'Computation of the transmitted psi wave'
  F=1;
  vmax=max(Vs2,Vf2)
  DO I=1,Nx
     x=X1+(I-1)*dx
     t0=arrivaltime(h,x,y,V1,Vpsi2)
     pp0=path(x,y,h,V1,Vpsi2,t0)     
     DO J=1,Nt
       t=T1+(j-1)*dt
       if (abs(aimag(pp0)).gt.1/Vmax) then
          !Computation of the head wave'
          thead=abs(y)*sqrt(1/Vpsi2**2-1/vmax**2)+h*sqrt(1/V1**2-1/vmax**2)+abs(x)/vmax;
           if((t.gt.thead).and.(t-2.*tdelay.lt.t0)) then
             if (t.ge.t0) then
                if (t-2.*tdelay.gt.thead) then
                   ddt=(t0-t+2.*tdelay)/Nint
                   tau0=t-2.*tdelay
                else
                   ddt=(t0-thead)/Nint
                   tau0=thead
                end if
             DO  k=1,Nint-1
                tau=tau0+(k-1)*ddt+ddt/2.
                src=source(t-tau)
                pp=path(-abs(x),y,h,V1,Vpsi2,tau)  
                pp=ima*aimag(pp)   
                kpsi2=zsqrt(pp**2+1/Vpsi2**2)
                Coeff=calccoeff_ap(pp)
                green=derivee(-abs(x),y,h,V1,Vpsi2,pp)/pi*ddt
                Ux(i,j)=Ux(i,j)+F*real(kpsi2*Coeff(4)*green)*src
                Uy(i,j)=Uy(i,j)+F*real(ima*pp*Coeff(4)*green)*src
             END DO
             tau=t0-ddt/2.
             src=source(t-tau)
             pp=path(-abs(x),y,h,V1,Vpsi2,tau)   
                pp=ima*aimag(pp)  
             kpsi2=zsqrt(pp**2+1/Vpsi2**2)
             Coeff=calccoeff_ap(pp)
             green=sqrt(t0**2-tau**2)*derivee(-abs(x),y,h,V1,Vpsi2,pp)*acos(1-ddt/t0)/pi*ddt
             Ux(i,j)=Ux(i,j)+F*real(kpsi2*Coeff(4)*green)*src
             Uy(i,j)=Uy(i,j)+F*real(ima*pp*Coeff(4)*green)*src   
          else
             if (t-2.*tdelay.gt.thead) then
                   ddt=2.*tdelay/Nint
                   tau0=t-2.*tdelay
                else
                   ddt=(t-thead)/Nint
                   tau0=thead
                end if
             DO  k=1,Nint
                tau=tau0+(k-1)*ddt+ddt/2.
                src=source(t-tau)
                pp=path(-abs(x),y,h,V1,Vpsi2,tau)  
                pp=ima*aimag(pp)   
                kpsi2=zsqrt(pp**2+1/Vpsi2**2)
                Coeff=calccoeff_ap(pp)
                green=derivee(-abs(x),y,h,V1,Vpsi2,pp)/pi*ddt
                Ux(i,j)=Ux(i,j)+F*real(kpsi2*Coeff(4)*green)*src
                Uy(i,j)=Uy(i,j)+F*real(ima*pp*Coeff(4)*green)*src
             END DO
          end if
       end if
    end if
       if (t.gt.t0) then
          if (t-2.*tdelay.gt.t0) then
             ddt=2.*tdelay/Nint
             tau0=t-2.*tdelay
          do k=1,Nint
             tau=tau0+(k-1)*ddt+ddt/2.
             src=source(t-tau)
             pp=path(-abs(x),y,h,V1,Vpsi2,tau)   
             kpsi2=zsqrt(pp**2+1/Vpsi2**2)
             Coeff=calccoeff_ap(pp)
             green=derivee(-abs(x),y,h,V1,Vpsi2,pp)/pi*ddt
             Ux(i,j)=Ux(i,j)+F*real(kpsi2*Coeff(4)*green)*src
             Uy(i,j)=Uy(i,j)+F*real(ima*pp*Coeff(4)*green)*src
          end do
       else
          ddt=(t-t0)/Nint
          tau0=t0
          tau=tau0+ddt/2.
          src=source(t-tau)
          pp=path(-abs(x),y,h,V1,Vpsi2,tau)   
          kpsi2=zsqrt(pp**2+1/Vpsi2**2)
          Coeff=calccoeff_ap(pp)
          green=sqrt(tau**2-t0**2)*derivee(-abs(x),y,h,V1,Vpsi2,pp)*dacosh(1+ddt/t0)/pi
          Ux(i,j)=Ux(i,j)+F*real(kpsi2*Coeff(4)*green)*src
          Uy(i,j)=Uy(i,j)+F*real(ima*pp*Coeff(4)*green)*src
          do k=2,Nint
             tau=tau0+(k-1)*ddt+ddt/2.
             src=source(t-tau)
             pp=path(-abs(x),y,h,V1,Vpsi2,tau)   
             kpsi2=zsqrt(pp**2+1/Vpsi2**2)
             Coeff=calccoeff_ap(pp)
             green=derivee(-abs(x),y,h,V1,Vpsi2,pp)/pi*ddt
             Ux(i,j)=Ux(i,j)+F*real(kpsi2*Coeff(4)*green)*src
             Uy(i,j)=Uy(i,j)+F*real(ima*pp*Coeff(4)*green)*src
          end do
       end if
    end if
     END DO
  END DO
END SUBROUTINE sub_transmitpsi_ap
